from ftw.usermanagement import user_management_factory as _
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.statusmessages.interfaces import IStatusMessage


class GroupDelete(BrowserView):
    """Get a list of groups from REQUEST to delete"""

    template = ViewPageTemplateFile('group_delete.pt')

    def __call__(self):
        return self.template()

    def delete(self):
        """Delete groups"""

        groupids = self.request.get('groupids', [])
        groupids = isinstance(groupids, list) and groupids or [groupids]

        try:
            getToolByName(self, 'portal_groups').removeGroups(groupids)
        except KeyError:
            IStatusMessage(self.request).addStatusMessage(
                _(u'There was an error while deleting a group.'), type="error")
            return

        IStatusMessage(self.request).addStatusMessage(
            _(u'Group(s) deleted.'), type="info")



from ftw.testing import MockTestCase
from ftw.usermanagement.testing import USERMANAGEMENT_ZCML_LAYER
from ftw.usermanagement.browser.base_listing import BaseManagementTableSource


class BaseManagementTableSourceTests(MockTestCase):

    layer = USERMANAGEMENT_ZCML_LAYER

    def setUp(self):
        super(BaseManagementTableSourceTests, self).setUp()

        self.request = {}
        self.config = self.mocker.mock(count=False)

    def test_validate_base_query_valid(self):

        source = BaseManagementTableSource(self.config, self.request)
        self.replay()

        result = source.validate_base_query({'query': 'valid'})

        self.assertEquals(result, {'query': 'valid'})

    def test_validate_base_query_invalid(self):

        source = BaseManagementTableSource(self.config, self.request)
        self.replay()

        self.assertRaises(
            ValueError,
            source.validate_base_query,
            ['query'])

    def test_extend_query_with_ordering(self):

        source = BaseManagementTableSource(self.config, self.request)
        self.replay()

        result = source.extend_query_with_ordering({'query': 'valid'})

        self.assertEquals(result, {'query': 'valid'})

    def test_extend_query_with_textfilter(self):

        source = BaseManagementTableSource(self.config, self.request)
        self.replay()

        result = source.extend_query_with_textfilter({}, 'filter text')

        self.assertEquals(result, {'filter_text': 'filter text'})

    def test_extend_query_with_batching(self):

        self.expect(self.config.batching_enabled).result(True)
        self.expect(self.config.lazy).result(False)
        self.expect(self.config.batching_pagesize).result(3)
        self.expect(self.config.batching_current_page).result(4)

        source = BaseManagementTableSource(self.config, self.request)

        self.replay()

        result = source.extend_query_with_batching({})

        self.assertEquals(
            result,
            {
                'pagesize': 3,
                'current_page': 4,
                'batching': True,
            })

    def test_extend_query_with_batching_is_lazy(self):

        self.expect(self.config.batching_enabled).result(True)
        self.expect(self.config.lazy).result(True)

        source = BaseManagementTableSource(self.config, self.request)

        self.replay()

        result = source.extend_query_with_batching({})

        self.assertEquals(result, {'batching': False})

    def test_extend_query_with_batching_no_batching(self):

        self.expect(self.config.batching_enabled).result(False)
        self.expect(self.config.lazy).result(False)

        source = BaseManagementTableSource(self.config, self.request)

        self.replay()

        result = source.extend_query_with_batching({})

        self.assertEquals(result, {'batching': False})



# coding=UTF-8
from ftw.testing import MockTestCase
from mocker import ANY, KWARGS
from ftw.usermanagement.testing import USERMANAGEMENT_ZCML_LAYER
from ftw.usermanagement.browser.user.user_notify import UserNotify
from Products.statusmessages.interfaces import IStatusMessage
from zope.interface import Interface
from Products.CMFCore.interfaces import IPropertiesTool


class GetOptionsTests(MockTestCase):

    layer = USERMANAGEMENT_ZCML_LAYER

    def setUp(self):
        super(GetOptionsTests, self).setUp()

        self.request = {}
        self.context = self.stub()

        self.notify = self.mocker.patch(UserNotify(self.context, self.request))
        self.expect(
            self.notify.reset_password(ANY)).result('12345').count(0, None)

        self.member = self.mocker.mock(count=False)
        self.expect(self.member.getProperty('email')).result('james@bond.ch')
        self.expect(
            self.member.getProperty('fullname', ANY)).result('James Bond')
        self.expect(self.member.id).result('user_id_1')

        self.mtool = self.mocker.mock(count=False)
        self.mock_tool(self.mtool, 'portal_membership')

        self.urltool = self.mocker.mock(count=False)
        self.mock_tool(self.urltool, 'portal_url')
        self.expect(
            self.urltool.getPortalObject().Title()).result('Pörtal Title')

        self.ptool = self.mocker.mock(count=False)
        self.mock_utility(self.ptool, IPropertiesTool)
        self.expect(self.ptool.email_from_address).result('contact@test.ch')

    def test_no_member(self):

        self.expect(self.mtool.getMemberById(ANY)).result(None)

        self.replay()

        result = self.notify.get_options('user_id', False)

        self.assertEquals(result, False)

    def test_with_member_no_pw_reset(self):

        self.expect(self.mtool.getMemberById(ANY)).result(self.member)

        self.replay()

        result = self.notify.get_options('user_id', False)

        self.assertEquals(result['email'], 'james@bond.ch')
        self.assertEquals(result['username'], 'user_id_1')
        self.assertEquals(result['fullname'], 'James Bond')
        self.assertEquals(result['site_title'], 'Pörtal Title')
        self.assertEquals(result['contact_email'], 'contact@test.ch')
        self.assertEquals(result['pw'], None)
        self.assertEquals(result['reset_pw'], False)

    def test_with_member_and_pw_reset(self):

        self.expect(self.mtool.getMemberById(ANY)).result(self.member)

        self.replay()

        result = self.notify.get_options('user_id', True)

        self.assertEquals(result['email'], 'james@bond.ch')
        self.assertEquals(result['username'], 'user_id_1')
        self.assertEquals(result['fullname'], 'James Bond')
        self.assertEquals(result['site_title'], 'Pörtal Title')
        self.assertEquals(result['contact_email'], 'contact@test.ch')
        self.assertEquals(result['pw'], '12345')
        self.assertEquals(result['reset_pw'], True)


class ResetPasswordTests(MockTestCase):

    layer = USERMANAGEMENT_ZCML_LAYER

    def setUp(self):
        super(ResetPasswordTests, self).setUp()

        self.request = {}
        self.context = self.stub()

        self.notify = self.mocker.patch(UserNotify(self.context, self.request))

        self.rolemaker = self.mocker.mock()
        self.expect(self.rolemaker.getRolesForPrincipal(ANY)).result(['role'])

        self.plugins = [['plugin_id', self.rolemaker]]

        self.acl_user_folder = []
        self.acltool = self.mocker.mock(count=False)
        self.mock_tool(self.acltool, 'acl_users')
        self.expect(self.acltool.plugins.listPlugins(ANY)).result(self.plugins)
        self.expect(self.acltool.getUserById(ANY)).call(lambda x: x)
        self.expect(self.acltool.userFolderEditUser(
            ANY, ANY, ANY, ANY, KWARGS)).call(
                lambda x, y, z, a, REQUEST: self.acl_user_folder.append(
                    {'member': x, 'password': y, 'roles': z}))

        self.rtool = self.mocker.mock(count=False)
        self.mock_tool(self.rtool, 'portal_registration')
        self.expect(self.rtool.generatePassword()).result('12345')

        self.member = self.mocker.mock(count=False)
        self.expect(self.member.id).result('user_id_1')
        self.expect(self.member.getDomains()).result('domains')

    def test_reset_pw(self):

        self.replay()

        result = self.notify.reset_password(self.member)

        self.assertEquals(result, '12345')
        self.assertEquals(len(self.acl_user_folder), 1)
        self.assertEquals(self.acl_user_folder[0]['member'], 'user_id_1')
        self.assertEquals(self.acl_user_folder[0]['password'], '12345')
        self.assertEquals(self.acl_user_folder[0]['roles'], ['role'])


class GetMailObjectTests(MockTestCase):

    layer = USERMANAGEMENT_ZCML_LAYER

    def setUp(self):
        super(GetMailObjectTests, self).setUp()

        self.notify_view = self.mocker.mock()
        self.expect(self.notify_view(KWARGS)).result(u'Body Täxt')

        self.request = {}
        self.context = self.stub()
        self.expect(
            self.context.unrestrictedTraverse(ANY)).result(self.notify_view)

        self.notify = self.mocker.patch(UserNotify(self.context, self.request))

        self.urltool = self.mocker.mock(count=False)
        self.mock_tool(self.urltool, 'portal_url')
        self.expect(
            self.urltool.getPortalObject().Title()).result('Pörtal Title')

        self.ptool = self.mocker.mock(count=False)
        self.mock_utility(self.ptool, IPropertiesTool)
        self.expect(self.ptool.email_from_name).result('James Bönd')
        self.expect(self.ptool.email_from_address).result('james@bond.ch')

        self.options = {'email': 'ted@mos.ch'}

    def test_get_mail_object(self):

        self.replay()

        result = self.notify.get_mail_object(self.options)

        self.assertEquals(
            result._headers[0][1], 'multipart/alternative')
        self.assertEquals(
            result._headers[2][1]._chunks[0][0], 'Welcome on Pörtal Title')
        self.assertEquals(
            result._headers[3][1], 'James Bönd<james@bond.ch>')
        self.assertEquals(
            result._headers[4][1], 'ted@mos.ch')

class ValidateTests(MockTestCase):

    layer = USERMANAGEMENT_ZCML_LAYER

    def setUp(self):
        super(ValidateTests, self).setUp()

        self.mailhost = self.mocker.mock()

        self.request = {}
        self.context = self.stub()
        self.expect(
            self.context.unrestrictedTraverse(ANY)).result(self.mailhost)

        self.notify = self.mocker.patch(UserNotify(self.context, self.request))

        self.statusmsg = self.mocker.mock(count=False)
        self.message_cache = self.create_dummy()
        self.expect(self.statusmsg(ANY).addStatusMessage(ANY, type=ANY)).call(
            lambda msg, type: setattr(self.message_cache, type, msg))
        self.mock_adapter(self.statusmsg, IStatusMessage, (Interface, ))

    def test_valid(self):

        options = {
            'email': 'user_1',
        }

        self.expect(self.mailhost.mailhost_warning()).result(False)

        self.replay()

        result = self.notify.validate(options)

        self.assertEquals(result, True)

    def test_no_options(self):

        options = False

        self.replay()

        result = self.notify.validate(options)

        self.assertEquals(result, False)
        self.assertEquals(
            self.message_cache.error,
            u'There was an error to look up a user')

    def test_no_mailhost(self):

        options = {
            'email': 'user_1',
        }

        self.expect(self.mailhost.mailhost_warning()).result(True)

        self.replay()

        result = self.notify.validate(options)

        self.assertEquals(result, False)
        self.assertEquals(
            self.message_cache.error,
            u'No mailhost defined. contact site Administrator.')

    def test_no_email(self):

        options = {
            'email': '',
        }

        self.expect(self.mailhost.mailhost_warning()).result(False)

        self.replay()

        result = self.notify.validate(options)

        self.assertEquals(result, False)
        self.assertEquals(self.message_cache.error, u'no_email_for_user')

class SendNotificationTest(MockTestCase):

    layer = USERMANAGEMENT_ZCML_LAYER

    def setUp(self):
        super(SendNotificationTest, self).setUp()

        self.request = {}
        self.context = self.stub()

        self.hosttool = self.mocker.mock(count=False)
        self.mock_tool(self.hosttool, 'MailHost')
        self.expect(self.hosttool.send(ANY, mto=ANY)).result(True)

        self.options = {'fullname': 'Üser 1', 'email': 'mail_1'}

        self.notify = self.mocker.patch(UserNotify(self.context, self.request))
        self.expect(self.notify.get_options(ANY, ANY)).result(self.options)

        self.statusmsg = self.mocker.mock(count=False)
        self.message_cache = self.create_dummy()
        self.expect(self.statusmsg(ANY).addStatusMessage(ANY, type=ANY)).call(
            lambda msg, type: setattr(self.message_cache, type, msg))
        self.mock_adapter(self.statusmsg, IStatusMessage, (Interface, ))

    def test_not_valid(self):

        self.expect(self.notify.validate(ANY)).result(False)

        self.replay()

        result = self.notify.send_user_notification('uid')

        self.assertEquals(result, False)

    def test_valid(self):

        self.expect(self.notify.validate(ANY)).result(True)
        self.expect(self.notify.get_mail_object(ANY)).result('mail')

        self.replay()

        self.notify.send_user_notification('uid')

        self.assertEquals(
            self.message_cache.info,
            u"text_usermanagament_user_notified",
        )


class CallTests(MockTestCase):

    layer = USERMANAGEMENT_ZCML_LAYER

    def setUp(self):
        super(CallTests, self).setUp()

        self.request = {}
        self.context = self.stub()

        self.notification_pool = []

        self.notify = self.mocker.patch(UserNotify(self.context, self.request))
        self.expect(self.notify.send_user_notification(ANY, ANY)).call(
            lambda x, y: self.notification_pool.append({x: y})).count(0, None)

    def test_no_userids(self):

        self.replay()

        self.notify()

        self.assertEquals(len(self.notification_pool), 0)

    def test_one_userid(self):

        self.request['userids'] = 'user_1'

        self.replay()

        self.notify()

        self.assertEquals(len(self.notification_pool), 1)
        self.assertEquals(self.notification_pool[0].get('user_1'), False)

    def test_more_userids(self):

        self.request['userids'] = ['user_1', 'user_2', 'user_3']

        self.replay()

        self.notify()

        self.assertEquals(len(self.notification_pool), 3)
        self.assertEquals(self.notification_pool[0].get('user_1'), False)
        self.assertEquals(self.notification_pool[1].get('user_2'), False)
        self.assertEquals(self.notification_pool[2].get('user_3'), False)

    def test_with_pw_reset(self):

        self.request['userids'] = ['user_1']
        self.request['reset_pw'] = True

        self.replay()

        self.notify()

        self.assertEquals(self.notification_pool[0].get('user_1'), True)



# coding=UTF-8
from ftw.testing import MockTestCase
from mocker import ANY
from ftw.usermanagement.testing import USERMANAGEMENT_ZCML_LAYER
from ftw.usermanagement.browser.group.group_membership import GroupMembership
from Products.statusmessages.interfaces import IStatusMessage
from zope.interface import Interface


class DisplayUsersTests(MockTestCase):

    layer = USERMANAGEMENT_ZCML_LAYER

    def setUp(self):
        super(DisplayUsersTests, self).setUp()

        self.request = {}
        self.context = self.stub()

        self.group_membership = self.mocker.patch(
            GroupMembership(self.context, self.request))
        self.expect(self.group_membership.get_users(ANY)).call(lambda x: x)

        self.member_1 = self.mocker.mock(count=False)
        self.expect(
            self.member_1.getProperty('fullname', ANY)).result('Jämes Bond')
        self.expect(self.member_1.getUserId()).result('member_1')

        self.member_2 = self.mocker.mock(count=False)
        self.expect(
            self.member_2.getProperty('fullname', ANY)).result('')
        self.expect(self.member_2.getUserId()).result('member_2')

    def test_no_users(self):

        self.expect(
            self.group_membership.membershipSearch(searchGroups=ANY)).result([])

        self.replay()

        result = self.group_membership.get_display_users([])

        self.assertEquals(result, [])

    def test_with_current_users(self):

        self.expect(
            self.group_membership.membershipSearch(searchGroups=ANY)).result(
                [self.member_1, self.member_2])

        self.replay()

        result = self.group_membership.get_display_users(['member_2'])

        self.assertEquals(
            result,
            [{
                'userid': 'member_1',
                'name': 'Jämes Bond',
                'is_member_of': False,
            },
            {
                'userid': 'member_2',
                'name': 'member_2',
                'is_member_of': True,
            }],
        )

    def test_no_current_users(self):

        self.expect(
            self.group_membership.membershipSearch(searchGroups=ANY)).result(
                [self.member_1, self.member_2])

        self.replay()

        result = self.group_membership.get_display_users([])

        self.assertEquals(
            result,
            [{
                'userid': 'member_1',
                'name': 'Jämes Bond',
                'is_member_of': False,
            },
            {
                'userid': 'member_2',
                'name': 'member_2',
                'is_member_of': False,
            }],
        )


class ReplaceGroupMembersTests(MockTestCase):

    layer = USERMANAGEMENT_ZCML_LAYER

    def setUp(self):
        super(ReplaceGroupMembersTests, self).setUp()

        self.request = {}

        self.context = self.stub()
        self.expect(self.context.REQUEST).result(self.request)

        self.gtool = self.mocker.mock(count=False)
        self.mock_tool(self.gtool, 'portal_groups')

        self.statusmsg = self.mocker.mock(count=False)
        self.message_cache = self.create_dummy()
        self.expect(self.statusmsg(ANY).addStatusMessage(ANY, type=ANY)).call(
            lambda msg, type: setattr(self.message_cache, type, msg))
        self.mock_adapter(self.statusmsg, IStatusMessage, (Interface, ))

        self.group_membership = self.mocker.patch(
            GroupMembership(self.context, self.request))
        self.expect(self.group_membership.get_users(ANY)).call(lambda x: x)

        self.removed_users = []
        self.added_users = []

        self.expect(self.gtool.removePrincipalFromGroup(
            ANY, ANY, ANY)).call(lambda x, y, z: self.removed_users.append(x))
        self.expect(self.gtool.addPrincipalToGroup(
            ANY, ANY, ANY)).call(lambda x, y, z: self.added_users.append(x))

    def test_nothing(self):

        self.replay()

        result = self.group_membership.replace_group_members([], [])

        self.assertEquals(result, True)
        self.assertEquals(self.message_cache.info, u'Changes made.')
        self.assertTrue(len(self.removed_users) == 0)
        self.assertTrue(len(self.added_users) == 0)

    def test_add(self):

        self.replay()

        result = self.group_membership.replace_group_members(
            [], ['user1', 'user2'])

        self.assertEquals(result, True)
        self.assertEquals(self.message_cache.info, u'Changes made.')
        self.assertTrue(len(self.removed_users) == 0)
        self.assertTrue(len(self.added_users) == 2)

    def test_remove(self):

        self.replay()

        result = self.group_membership.replace_group_members(
            ['user1', 'user2'], [])

        self.assertEquals(result, True)
        self.assertEquals(self.message_cache.info, u'Changes made.')
        self.assertTrue(len(self.removed_users) == 2)
        self.assertTrue(len(self.added_users) == 0)

    def test_remove_and_add(self):

        self.replay()

        result = self.group_membership.replace_group_members(
            ['user1', 'user2'], ['user3', 'user4'])

        self.assertEquals(result, True)
        self.assertEquals(self.message_cache.info, u'Changes made.')
        self.assertTrue(len(self.removed_users) == 2)
        self.assertTrue(len(self.added_users) == 2)

    def test_leave_all(self):

        self.replay()

        result = self.group_membership.replace_group_members(
            ['user1', 'user2'], ['user1', 'user2'])

        self.assertEquals(result, True)
        self.assertEquals(self.message_cache.info, u'Changes made.')
        self.assertTrue(len(self.removed_users) == 0)
        self.assertTrue(len(self.added_users) == 0)


class CallTests(MockTestCase):

    layer = USERMANAGEMENT_ZCML_LAYER

    def setUp(self):
        super(CallTests, self).setUp()

        self.request = {}

        self.context = self.stub()
        self.expect(self.context.REQUEST).result(self.request)

    def test_no_group_id(self):

        request = self.mocker.proxy(self.request)
        self.expect(request.form).result({})

        self.replay()

        result = GroupMembership(self.context, request)()

        self.assertEquals(result, 'No group selected')

    def test_form_submitted(self):

        self.request['group_id'] = "Group_1"
        request = self.mocker.proxy(self.request)
        self.expect(request.form).result({'form.submitted': True})

        group_membership = self.mocker.patch(
            GroupMembership(self.context, request))
        self.expect(
            group_membership.replace_group_members(ANY, ANY)).result('replace')

        self.replay()

        result = group_membership()

        self.assertEquals(result, 'replace')

    def test_form_not_submitted(self):

        self.request['group_id'] = "Group_1"
        request = self.mocker.proxy(self.request)
        self.expect(request.form).result({'form.submitted': False})

        group_membership = self.mocker.patch(
            GroupMembership(self.context, request))
        self.expect(
            group_membership.render(ANY)).result('render')

        self.replay()

        result = group_membership()

        self.assertEquals(result, 'render')


class GetUsersTests(MockTestCase):

    layer = USERMANAGEMENT_ZCML_LAYER

    def setUp(self):
        super(GetUsersTests, self).setUp()

        self.request = {}

        self.context = self.stub()
        self.expect(self.context.REQUEST).result(self.request)

        self.gtool = self.mocker.mock(count=False)
        self.mock_tool(self.gtool, 'portal_groups')

        self.mtool = self.mocker.mock(count=False)
        self.mock_tool(self.mtool, 'portal_membership')

        self.member_1 = self.mocker.mock(count=False)
        self.expect(
            self.member_1.getProperty('fullname', ANY)).result('Jämes Bond')

        self.member_2 = self.mocker.mock(count=False)
        self.expect(
            self.member_2.getProperty('fullname', ANY)).result('')

    def test_no_members(self):

        self.expect(self.gtool.getGroupMembers(ANY)).result([])

        self.replay()

        gm = GroupMembership(self.context, self.request)
        result = gm.get_users('no_group')

        # No users
        self.assertEquals(result, [])

    def test_with_members(self):

        self.expect(self.gtool.getGroupMembers(ANY)).result(
            ['fullname', 'no_fullname', 'admin'])

        # No member, just id
        self.expect(self.mtool.getMemberById('admin')).result(None)

        # Member with fullname
        self.expect(
            self.mtool.getMemberById('fullname')).result(self.member_1)

        # Member without fullname
        self.expect(self.mtool.getMemberById(
            'no_fullname')).result(self.member_2)

        self.replay()

        gm = GroupMembership(self.context, self.request)
        result = gm.get_users('group_id')

        # Sorted list with userids
        self.assertEquals(result, ['fullname', 'admin', 'no_fullname'])



from ftw.testing import MockTestCase
from ftw.usermanagement.browser.user.user_register import UserRegister
from ftw.usermanagement.testing import USERMANAGEMENT_ZCML_LAYER
from mocker import ANY
from Products.CMFCore.interfaces import ISiteRoot
from Products.statusmessages.interfaces import IStatusMessage
from zope.interface import Interface


class ValidateTests(MockTestCase):

    layer = USERMANAGEMENT_ZCML_LAYER

    def setUp(self):
        super(ValidateTests, self).setUp()

        self.request = {}

        self.context = self.stub()
        self.expect(self.context.REQUEST).result(self.request)

        self.rtool = self.mocker.mock(count=False)
        self.mock_tool(self.rtool, 'portal_registration')

        self.site_root = self.mocker.mock(count=False)
        self.mock_utility(self.site_root, ISiteRoot)
        self.expect(self.site_root.getId()).result('portal_id')

        self.statusmsg = self.mocker.mock(count=False)
        self.message_cache = []
        self.expect(self.statusmsg(ANY).addStatusMessage(ANY, type=ANY)).call(
            lambda msg, type: self.message_cache.append({type: msg}))
        self.mock_adapter(self.statusmsg, IStatusMessage, (Interface, ))

    def test_no_errors(self):

        self.expect(self.rtool.isMemberIdAllowed(ANY)).result(True)
        self.expect(self.rtool.isValidEmail(ANY)).result(True)

        self.replay()

        data = {
            'email': 'valid_email',
            'firstname': 'valid_firstname',
            'lastname': 'valid_lastname',
            'username': 'valid_username',
            }

        result = UserRegister(
            self.context, self.request).validate_registration(data)

        self.assertEquals(result, True)

    def test_empty_firstname_lastname_email(self):

        self.expect(self.rtool.isMemberIdAllowed(ANY)).result(True)
        self.expect(self.rtool.isValidEmail(ANY)).result(True)

        self.replay()

        data = {
            'email': '',
            'firstname': '',
            'lastname': '',
            }

        result = UserRegister(
            self.context, self.request).validate_registration(data)

        self.assertEquals(result, False)
        self.assertTrue(len(self.message_cache) == 3)
        self.assertTrue(
            self.message_cache[0], {'error': u'text_missing_firstname'})
        self.assertTrue(
            self.message_cache[1], {'error': u'text_missing_lastname'})
        self.assertTrue(
            self.message_cache[2], {'error': u'text_missing_email'})

    def test_reserved_username(self):

        self.expect(self.rtool.isMemberIdAllowed(ANY)).result(True)
        self.expect(self.rtool.isValidEmail(ANY)).result(True)

        self.replay()

        data = {
            'email': 'valid_email',
            'firstname': 'valid_firstname',
            'lastname': 'valid_lastname',
            'username': 'portal_id',
        }

        result = UserRegister(
            self.context, self.request).validate_registration(data)

        self.assertEquals(result, False)
        self.assertTrue(len(self.message_cache) == 1)
        self.assertTrue(
            self.message_cache[0],
            {'error': "This username is reserved. Please "
            "choose a different name."})

    def test_invalid_username(self):

        self.expect(self.rtool.isMemberIdAllowed(ANY)).result(False)
        self.expect(self.rtool.isValidEmail(ANY)).result(True)

        self.replay()

        data = {
            'email': 'valid_email',
            'firstname': 'valid_firstname',
            'lastname': 'valid_lastname',
            'username': 'invalid_username',
        }

        result = UserRegister(
            self.context, self.request).validate_registration(data)

        self.assertEquals(result, False)
        self.assertTrue(len(self.message_cache) == 1)
        self.assertTrue(
            self.message_cache[0],
            {'error': u"The login name you selected is already"
            "in use or is not valid. Please choose another."})

    def test_invalid_email(self):

        self.expect(self.rtool.isMemberIdAllowed(ANY)).result(True)
        self.expect(self.rtool.isValidEmail(ANY)).result(False)

        self.replay()

        data = {
            'email': 'invalid_email',
            'firstname': 'valid_firstname',
            'lastname': 'valid_lastname',
            'username': 'valid_username',
        }

        result = UserRegister(
            self.context, self.request).validate_registration(data)

        self.assertEquals(result, False)
        self.assertTrue(len(self.message_cache) == 1)
        self.assertTrue(
            self.message_cache[0],
            {'error': u'You must enter a valid email address.'})


class RegisterTests(MockTestCase):

    layer = USERMANAGEMENT_ZCML_LAYER

    def setUp(self):
        super(RegisterTests, self).setUp()

        self.request = {}

        self.context = self.stub()
        self.expect(self.context.REQUEST).result(self.request)

        self.mtool = self.mocker.mock(count=False)
        self.mock_tool(self.mtool, 'portal_membership')
        self.expect(self.mtool.memberareaCreationFlag(ANY)).result(True)
        self.expect(self.mtool.createMemberarea(ANY)).result(True)

        self.rtool = self.mocker.mock(count=False)
        self.mock_tool(self.rtool, 'portal_registration')
        self.expect(self.rtool.generatePassword()).result('12345')
        self.expect(self.rtool.addMember(
            'attribute_error',
            ANY,
            properties=ANY,
            REQUEST=ANY)).throw(AttributeError)
        self.expect(self.rtool.addMember(
            'value_error',
            ANY,
            properties=ANY,
            REQUEST=ANY)).throw(ValueError)
        self.expect(self.rtool.addMember(
            'ok',
            ANY,
            properties=ANY,
            REQUEST=ANY)).result(True)

        self.statusmsg = self.mocker.mock(count=False)
        self.message_cache = self.create_dummy()
        self.expect(self.statusmsg(ANY).addStatusMessage(ANY, type=ANY)).call(
            lambda msg, type: setattr(self.message_cache, type, msg))
        self.mock_adapter(self.statusmsg, IStatusMessage, (Interface, ))

    def test_ok(self):

        self.request['email'] = 'ok'

        register_obj = self.mocker.patch(
            UserRegister(self.context, self.request))

        self.expect(register_obj.validate_registration(ANY)).result(True)

        self.replay()

        result = register_obj.register()

        self.assertEquals(result, True)
        self.assertEquals(
            self.message_cache.__dict__.get('info'), u'User added.')

    def test_value_error(self):

        self.request['email'] = 'value_error'

        register_obj = self.mocker.patch(
            UserRegister(self.context, self.request))

        self.expect(register_obj.validate_registration(ANY)).result(True)

        self.replay()

        result = register_obj.register()

        self.assertEquals(result, False)
        self.assertEquals(
            type(self.message_cache.__dict__.get('error')), ValueError)

    def test_attribute_error(self):

        self.request['email'] = 'attribute_error'

        register_obj = self.mocker.patch(
            UserRegister(self.context, self.request))

        self.expect(register_obj.validate_registration(ANY)).result(True)

        self.replay()

        result = register_obj.register()

        self.assertEquals(result, False)
        self.assertEquals(
            type(self.message_cache.__dict__.get('error')), AttributeError)

    def test_validation_error(self):

        self.request['email'] = 'ok'

        register_obj = self.mocker.patch(
            UserRegister(self.context, self.request))

        self.expect(register_obj.validate_registration(ANY)).result(False)

        self.replay()

        result = register_obj.register()

        self.assertEquals(result, False)
        self.assertTrue(len(self.message_cache.__dict__) == 0)



from ftw.usermanagement import user_management_factory as _
from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from Products.statusmessages.interfaces import IStatusMessage
from zope.component import getUtility

ERROR_MSG = {
    'no_firstname': _(u'text_missing_firstname'),
    'no_lastname': _(u'text_missing_lastname'),
    'no_email': _(u'text_missing_email'),
    'reserved_uid': _(u"This username is reserved. Please "
        "choose a different name."),
    'invalid_uid': _(u"The login name you selected is already"
            "in use or is not valid. Please choose another."),
    'invalid_email': _(u'You must enter a valid email address.'),
}


class UserRegister(BrowserView):
    """Validate and register a new member
    """

    def __call__(self):
        return self.register()

    def register(self):
        """Register the new Member
        """
        registration = getToolByName(self.context, 'portal_registration')
        mtool = getToolByName(self, 'portal_membership')
        data = self._get_register_values()

        if not self.validate_registration(data):
            return False

        try:
            registration.addMember(
                data.get('username'),
                self._get_password(),
                properties=data,
                REQUEST=self.request)
        except (AttributeError, ValueError), err:
            self._add_statusmessage(err, "error")
            return False

        if mtool.memberareaCreationFlag:
            # Create the user's member area
            mtool.createMemberarea(data.get('username'))

        self._add_statusmessage(_(u"User added."), 'info')

        return True

    def validate_registration(self, data):
        """ Validate the registration
        """

        errors = {}
        portal = getUtility(ISiteRoot)
        registration = getToolByName(self.context, 'portal_registration')

        if not data.get('firstname'):
            errors['firstname'] = ERROR_MSG.get('no_firstname')

        if not data.get('lastname'):
            errors['lastname'] = ERROR_MSG.get('no_lastname')

        if not data.get('email'):
            errors['email'] = ERROR_MSG.get('no_email')

        # Check if username is valid
        elif data.get('username') == portal.getId():
            errors['email'] = ERROR_MSG.get('reserved_uid')

        # Check if username is allowed
        elif not registration.isMemberIdAllowed(data.get('username')):
            errors['email'] = ERROR_MSG.get('invalid_uid')

        # Check for valid email-address
        elif not registration.isValidEmail(data.get('email')):
            errors['email'] = ERROR_MSG.get('invalid_email')

        for msg in errors.values():
            self._add_statusmessage(msg, "error")

        return not errors

    def _add_statusmessage(self, msg, mtype):
        """ Add a statusmessage
        """
        IStatusMessage(self.request).addStatusMessage(msg, type=mtype)

    def _get_password(self):
        """ Get given password or generate one
        """
        registration = getToolByName(self.context, 'portal_registration')

        return self.request.get('password', '') or \
            registration.generatePassword()

    def _get_register_values(self):
        """ Return the required values for a registration
        """
        firstname = self.request.get('firstname', '')
        lastname = self.request.get('lastname', '')
        email = self.request.get('email', '')
        username = email

        data = dict(
            username=username,
            firstname=firstname,
            lastname=lastname,
            fullname='%s %s' % (lastname, firstname),
            email=email)

        return data



from ftw.usermanagement import user_management_factory as _
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from Products.statusmessages.interfaces import IStatusMessage


class GroupAdd(BrowserView):

    def __call__(self):

        group_id = self.request.get('group_id', '')
        group_title = self.request.get('group_title', group_id)

        self.create_group(group_id, group_title)

    def create_group(self, group_id, group_title):
        """Creates a new group
        """
        gtool = getToolByName(self.context, 'portal_groups')

        if not self.validate_group(group_id):
            return False

        data = dict(title=group_title or group_id)
        gtool.addGroup(group_id, **data)

        self._add_statusmessage(_(u'text_group_created'), "info")

        return True

    def validate_group(self, group_id):
        """ Validates the input
        """
        registration = getToolByName(self.context, 'portal_registration')
        no_errors = True

        # check if group_id is allowed
        if not registration.isMemberIdAllowed(group_id):
            msg = _(u"The group id you selected is already"
                "in use or is not valid. Please choose another.")
            self._add_statusmessage(msg, "error")
            no_errors = False

        return no_errors

    def _add_statusmessage(self, msg, mtype):
        """ Add a statusmessage
        """
        IStatusMessage(self.request).addStatusMessage(msg, type=mtype)



from plone.app.testing import applyProfile
from plone.app.testing import IntegrationTesting
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import PLONE_FIXTURE
from plone.testing import Layer
from plone.testing import zca
from plone.testing.z2 import installProduct
from zope.configuration import xmlconfig


class UsermanagementZCMLLayer(Layer):
    """A layer which only sets up the zcml, but does not start a zope
    instance.
    """

    defaultBases = (zca.ZCML_DIRECTIVES, )

    def testSetUp(self):
        self['configurationContext'] = zca.stackConfigurationContext(
            self.get('configurationContext'))

    def testTearDown(self):
        del self['configurationContext']


USERMANAGEMENT_ZCML_LAYER = UsermanagementZCMLLayer()


class UsermanagementPloneLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE, )

    def setUpZope(self, app, configurationContext):
        # Load ZCML
        import ftw.usermanagement
        xmlconfig.file(
            'configure.zcml', ftw.usermanagement, context=configurationContext)
        installProduct(app, 'ftw.usermanagement')

        import ftw.tabbedview
        xmlconfig.file(
            'configure.zcml', ftw.tabbedview, context=configurationContext)
        installProduct(app, 'ftw.tabbedview')

        import ftw.table
        xmlconfig.file(
            'configure.zcml', ftw.table, context=configurationContext)
        installProduct(app, 'ftw.table')

    def setUpPloneSite(self, portal):
        # Install into Plone site using portal_setup
        applyProfile(portal, 'ftw.usermanagement:default')
        applyProfile(portal, 'ftw.tabbedview:default')
        applyProfile(portal, 'ftw.table:default')

UsermanagementPloneFixture = UsermanagementPloneLayer()
USERMANAGEMENT_PLONE_LAYER = IntegrationTesting(
    bases=(UsermanagementPloneFixture, ),
    name="ftw.usermanagement:Integration")



from ftw.testing import MockTestCase
from mocker import ANY, KWARGS
from ftw.usermanagement.testing import USERMANAGEMENT_ZCML_LAYER
from ftw.usermanagement.browser.user.user_delete import UserDelete
from Products.statusmessages.interfaces import IStatusMessage
from zope.interface import Interface


class DeleteUserTests(MockTestCase):

    layer = USERMANAGEMENT_ZCML_LAYER

    def setUp(self):
        super(DeleteUserTests, self).setUp()

        self.request = {}
        self.context = self.stub()

        self.mtool = self.mocker.mock(count=False)
        self.mock_tool(self.mtool, 'portal_membership')
        self.expect(self.mtool.deleteMembers(ANY, KWARGS)).result(True)

        self.statusmsg = self.mocker.mock(count=False)
        self.message_cache = self.create_dummy()
        self.expect(self.statusmsg(ANY).addStatusMessage(ANY, type=ANY)).call(
            lambda msg, type: setattr(self.message_cache, type, msg))
        self.mock_adapter(self.statusmsg, IStatusMessage, (Interface, ))

    def test_no_id(self):

        self.replay()

        UserDelete(self.context, self.request).delete()

        self.assertEquals(self.message_cache.info, u'text_user_deleted')

    def test_one_id(self):

        self.request['userids'] = 'userid'

        self.replay()

        UserDelete(self.context, self.request).delete()

        self.assertEquals(self.message_cache.info, u'text_user_deleted')

    def test_more_ids(self):

        self.request['userids'] = ['userid_1', 'userid_1']

        self.replay()

        UserDelete(self.context, self.request).delete()

        self.assertEquals(self.message_cache.info, u'text_user_deleted')



from ftw.usermanagement import user_management_factory as _
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import getUtility
from zope.i18n import translate
from ftw.usermanagement.browser.base_listing import \
    BaseListing, BaseManagementTableSource, BaseSearchResultExecutor
from ftw.table.interfaces import ITableSourceConfig
from zope.interface import implements
from ftw.usermanagement.interfaces import IUserManagementVocabularyFactory


class IUsersSourceConfig(ITableSourceConfig):
    """Marker interface for a TableSourceConfig interface"""


def checkbox(item, value):
    return '<input type="checkbox" name="userids" value="%s" />' % \
        item.get('userid', '')


def userpreflink(item, value):
    url = './@@user-information?userid=%s' % item.get('userid', '')
    return '<a href="%s">%s</a>' % (url, item.get('name', ''))


def link_group(item, value):

    group_link = '<a href="./user_membership?userid=%s">%s</a>' % \
                       (item.get('userid', ''), value)

    return group_link


class UserManagement(BaseListing):
    """A ftw.table based user management view
    """
    implements(IUsersSourceConfig)

    show_menu = False
    show_selects = True

    columns = (
        {'column': 'counter',
         'column_title': _(u'label_nr', default='Nr.'),
         },
        {'transform': checkbox},
        {'column': 'name',
         'column_title': _(u'label_name', default='Name'),
         'transform': userpreflink},
        {'column': 'userid',
         'column_title': _(u'label_userid', default='Userid'), },
        {'column': 'groups',
         'column_title': _(u'label_groups', default='Groups'),
         'transform': link_group})

    template = ViewPageTemplateFile('users.pt')

    def __init__(self, context, request):
        super(UserManagement, self).__init__(context, request)

        self.sort_order = 'ASC'
        self.batching_enabled = True
        self.lazy = False


class UsersTableSource(BaseManagementTableSource):

    def search_results(self, query):
        """Executes the query and returns a tuple of `results` and `length`.
        """
        return UsersSearchResultExecutor(self.config.context, query)()


class UsersSearchResultExecutor(BaseSearchResultExecutor):
    """ A fast implementation to search and display users

    This Executor just loads the data of users we see. This makes it very
    fast. If we use the filter, we need to load the data of all users,
    so it will be slower if we use him.
    """

    def __init__(self, context, query):
        super(UsersSearchResultExecutor, self).__init__(context, query)

        self.gtool = getToolByName(self.context, 'portal_groups')
        self._b_area = None

    def __call__(self):
        return self.get_results()

    def get_results(self):
        """ Search for users with the given paranters set in the query
        """
        mtool = getToolByName(self.context, 'portal_membership')
        users_factory = self._factory
        users_map = []

        for i, user in enumerate(users_factory):

            if not self._should_load_userdata(i):
                # If we should not load userdata, we add an empty user
                # and continue with the next item
                users_map.append(dict(
                    counter=i + 1,
                    name='',
                    userid=user.get('id'),
                    groups='',
                    ))
                continue

            # This is very expensive. We need the memberinfo because we
            # need the fullname. Thats not stored in acl_users.
            user_info = mtool.getMemberInfo(user.get('id'))
            if not user_info:
                continue

            if not self._match_obj_with_filter(
                user_info, ['username', 'fullname']):
                # if the filter does not match with the userdata,
                # we continue with the next user
                continue

            #We need to use the login name to get the group because a plone function
            # is decleared wrong
            users_map.append(dict(
                counter=i + 1,
                name=self._get_fullname(user_info),
                userid=user.get('id'),
                groups=self.get_group_names_of_user(user.get('login')),
                )
            )

        return users_map

    def get_group_names_of_user(self, loginname):
        """ Return all groupnames of a user as a string
        If we have no groups, we return a translated info string.
        """
        groups = []
        for group in self._get_group_objects_of_user(loginname):

            if group.getId() in ['AuthenticatedUsers']:
                continue

            groups.append(group.getGroupTitleOrName())

        groups = ', '.join(groups)

        if not groups:
            groups = translate(u'no_group',
                             domain='ftw.usermanagement',
                             context=self.context.REQUEST,
                             default=u'No Group').encode('utf-8')

        return groups

    @property
    def batch_start(self):
        """ Return the start of the batch
        """
        return self._batch_area.get('start', 0)

    @property
    def batch_end(self):
        """ Return the end of the batch
        """
        return self._batch_area.get('end', 0)

    @property
    def _batch_area(self):
        """ Return a dict with the showable area
        """
        if not self._b_area:
            pagesize = self.query['pagesize']
            current_page = self.query['current_page']
            self._b_area = {
                'start': pagesize * (current_page - 1),
                'end': pagesize * current_page,
                }

        return self._b_area

    @property
    def _factory(self):
        """ Return the principalsource factory
        """
        factory = getUtility(
            IUserManagementVocabularyFactory,
            name='plone.principalsource.Users')

        return factory(self.context)

    def _get_fullname(self, user_info):
        """ Get the fullname. If we have no fullname, we return the id
        """
        return user_info.get('fullname') and user_info.get('fullname') or \
            user_info.get('username')

    def _should_load_userdata(self, position):
        """ Is batching active and is the user in the area to display
        """

        # If we have nothing in the filter and the item is in the visible
        # area of the batch, then we load the userdata
        if not self.query.get('filter_text', '') and \
            (not self.query['batching'] or \
            (position >= self.batch_start and position < self.batch_end)):
            return True

        # If we have anything in the textfilter, we have to load the
        # data of all users. So we can filter
        if self.query.get('filter_text', ''):
            return True

        return False

    def _cleanup_groups(self, groups):
        """ Sort and filter the given list of groups
        """
        groups.sort(
            key=lambda x: x is not None and x.getGroupTitleOrName().lower())
        return filter(None, groups)

    def _get_group_objects_of_user(self, loginname):
        """ Return all groupobjects of a given user
        """

        #This Function requires the login name and not the userid
        groups = self.gtool.getGroupsByUserId(loginname)

        groups = self._cleanup_groups(groups)

        return groups



from plone.principalsource.source import PrincipalSourceBinder, PrincipalSource


class FtwPrincipalSource(PrincipalSource):
    """ Optimized PrincipalSource to get a faster user overview
    """

    def __iter__(self):
        seen = set()
        for result in self.acl_users.searchUsers():
            if result['id'] in seen:
                continue

            seen.add(result['id'])
            yield result


class FtwPrincipalSourceBinder(PrincipalSourceBinder):
    """Bind the principal source with either users or groups
    """

    def __call__(self, context):
        return FtwPrincipalSource(context, self.users, self.groups)


UsersVocabularyFactory = FtwPrincipalSourceBinder(users=True, groups=False)
GroupsVocabularyFactory = FtwPrincipalSourceBinder(users=False, groups=True)



from ftw.tabbedview.browser.listing import ListingView
from ftw.table.basesource import BaseTableSource


class BaseListing(ListingView):
    """BaseListing View for a tab in the tabbedview"""

    def __init__(self, context, request):
        super(BaseListing, self).__init__(context, request)

        self.table_options = None

    def get_base_query(self):
        return {}


class BaseManagementTableSource(BaseTableSource):

    def validate_base_query(self, query):
        """Validates and fixes the base query. Returns the query object.
        """
        if not isinstance(query, dict):
            raise ValueError('Expected a dict from get_base_query() of ' + \
                                 str(self.config))

        return query

    def extend_query_with_ordering(self, query):
        """Extends the given `query` with ordering information and returns
        the new query.
        """
        # We do not implement sorting because performance problems

        return query

    def extend_query_with_textfilter(self, query, text):
        """Extends the given `query` with text filters. This is only done when
        config's `filter_text` is set.
        """
        query['filter_text'] = text

        return query

    def extend_query_with_batching(self, query):
        """Extends the given `query` with batching filters and returns the
        new query. This method is only called when batching is enabled in
        the source config with the `batching_enabled` attribute.
        When `lazy` is set to `True` in the configuration, this method is
        not called.
        """
        if not self.config.batching_enabled or self.config.lazy:
            query['batching'] = False
            return query

        query['pagesize'] = self.config.batching_pagesize
        query['current_page'] = self.config.batching_current_page
        query['batching'] = self.config.batching_enabled

        return query

    def search_results(self, query):
        """Executes the query and returns a tuple of `results` and `length`.
        """
        return BaseSearchResultExecutor(self.config.context, query)()


class BaseSearchResultExecutor(object):
    """ Search and display groups
    """

    def __init__(self, context, query):
        self.context = context
        self.query = query

    def __call__(self):
        return self.get_results()

    def get_results(self):
        """ Return results as a dict
        """
        return {}

    def _match_obj_with_filter(self, obj, filter_attrs):
        """ Matches the filtertext with the user
        """
        text = self.query.get('filter_text', '').lower()

        if not text:
            return True

        values = []
        for arg in filter_attrs:
            values.append(obj.get(arg, '').lower())

        values = ' '.join(values)

        return text in values



from ftw.usermanagement import user_management_factory as _
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import queryMultiAdapter
from ftw.usermanagement.browser.base_listing import \
    BaseListing, BaseManagementTableSource, BaseSearchResultExecutor
from ftw.table.interfaces import ITableSourceConfig
from zope.interface import implements


class IGroupsSourceConfig(ITableSourceConfig):
    """Marker interface for a TableSourceConfig interface"""


def checkbox(item, value):
    return '<input type="checkbox" name="groupids" value="%s" />' % \
        item['group_id']


def link_group(item, value):

    group_link = '<a href="./group_membership?group_id=%s">%s</a>' % \
                       (item['group_id'], value)

    return group_link


class GroupManagement(BaseListing):
    """A ftw.table based user management view"""

    implements(IGroupsSourceConfig)

    show_menu = False

    columns = (
        {'column': 'counter',
         'column_title': _(u'label_nr', default='Nr.'),
         },
        {'transform': checkbox},
        {'column': 'group_title',
         'column_title': _(u'label_group_title', default='Title'),
         'transform': link_group},
        {'column': 'group_id',
         'column_title': _(u'label_group_id', default='Id'), },
        )

    template = ViewPageTemplateFile('groups.pt')


class GroupsTableSource(BaseManagementTableSource):

    def search_results(self, query):
        """Executes the query and returns a tuple of `results` and `length`.
        """
        return GroupsSearchResultExecutor(self.config.context, query)()


class GroupsSearchResultExecutor(BaseSearchResultExecutor):
    """ Search and display groups
    """

    def get_results(self):
        """ Get title and id of all groups
        """
        groupprefs = queryMultiAdapter((self.context, self.context.REQUEST),
                                         name=u'usergroup-groupprefs')
        groups = groupprefs.doSearch(searchString='')
        results = []

        for group in groups:
            if group['groupid'] in ['AuthenticatedUsers']:
                continue

            if not self._match_obj_with_filter(
                group, ['groupid', 'title']):
                continue

            results.append(dict(
                group_id=group['groupid'],
                group_title=group['title'], ),
            )

        results = self._sort_groups(results)

        return results

    def _sort_groups(self, group):
        """ Sort the group by title
        """
        group.sort(key=lambda x: x.get('group_title'))

        return group



from email.header import Header
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from ftw.usermanagement import user_management_factory as _
from Products.CMFCore.interfaces import IPropertiesTool
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from Products.PluggableAuthService.interfaces.plugins import IRolesPlugin
from Products.statusmessages.interfaces import IStatusMessage
from zope.component import getUtility
from zope.i18n import translate


class UserNotify(BrowserView):
    """Notify a registered user.
    Possible to reset password with notification
    """

    def __call__(self):
        userids = self.request.get('userids', [])
        reset_pw = self.request.get('reset_pw', False)

        if not isinstance(userids, list):
            userids = [userids]

        for userid in userids:
            self.send_user_notification(userid, reset_pw)

    def send_user_notification(self, userid, reset_pw=False):
        """Notifies a user, that he has been registered on the portal
        """
        options = self.get_options(userid, reset_pw)

        if not self.validate(options):
            return False

        mail = self.get_mail_object(options)

        # send the mail
        mh = getToolByName(self.context, 'MailHost')
        mh.send(mail, mto=options.get('email'))

        self._add_statusmessage(
            _(u'text_usermanagament_user_notified',
                default=u"User ${fullname} (${email}) has been notified",
                mapping={
                    'fullname': options.get('fullname').decode('utf-8'),
                    'email': options.get('email').decode('utf-8')}),
            'info')

    def validate(self, options):
        """ Validate the received data
        """
        if not options:
            self._add_statusmessage(
                _(u'There was an error to look up a user'),
                'error',
            )
            return False

        if not self._check_mailhost():
            self._add_statusmessage(
                _(u'No mailhost defined. contact site Administrator.'),
                'error')
            return False

        if not options.get('email'):
            self._add_statusmessage(_(
                u'no_email_for_user',
                default='For the user ${user} exist no email address',
                mapping=dict(user=options.get('username'))),
                'error',
            )
            return False
        return True

    def get_mail_object(self, options):
        """ Return a multipart mail object
        """
        title = self._get_site_title()
        body = self._get_html_body(options)

        mail = MIMEMultipart('alternative')
        mail['Subject'] = Header(
             self._get_subject(title.decode('utf-8')), 'utf-8')
        mail['From'] = '%s<%s>' % (
            self._get_contact_name(), self._get_contact_email())
        mail['To'] = options.get('email')
        mail.attach(MIMEText(body, 'html', 'utf-8'))

        return mail

    def reset_password(self, member):
        """ Reset and return the password for the given user
        """
        acl_users = getToolByName(self.context, 'acl_users')
        registration = getToolByName(self.context, 'portal_registration')

        rolemakers = acl_users.plugins.listPlugins(IRolesPlugin)
        all_assigned_roles = []

        # Get all assigned roles
        for rolemaker in rolemakers:
            all_assigned_roles.extend(
                rolemaker[1].getRolesForPrincipal(
                    acl_users.getUserById(member.id)))

        # Generate new password
        password = registration.generatePassword()

        # Update user properties
        acl_users.userFolderEditUser(
            member.id,
            password,
            all_assigned_roles,
            member.getDomains(),
            REQUEST=self.request)

        return password

    def get_options(self, userid, reset_pw):
        """ Return a map with all required infos for the mail template
        """
        mtool = getToolByName(self, 'portal_membership')
        member = mtool.getMemberById(userid)

        if not member:
            return False

        options = {}

        options['email'] = member.getProperty('email')
        options['username'] = member.id
        options['fullname'] = member.getProperty('fullname', member.id)
        options['site_title'] = self._get_site_title()
        options['contact_email'] = self._get_contact_email()
        options['pw'] = reset_pw and self.reset_password(member) or None
        options['reset_pw'] = reset_pw

        return options

    def _check_mailhost(self):
        """ Check for mailhost warnings. If we have any, we abort the
        notification
        """
        mailhost = self.context.unrestrictedTraverse('@@overview-controlpanel')
        if mailhost.mailhost_warning():
            return False
        return True

    def _get_contact_email(self):
        """ Return the contact email from the request or propertiestool
        """
        properties = getUtility(IPropertiesTool)
        contact_mail = self.request.get('contact.email',
                properties.email_from_address)

        return contact_mail

    def _get_contact_name(self):
        """ Return the contact name from propertiestool
        """
        properties = getUtility(IPropertiesTool)
        return properties.email_from_name

    def _get_site_title(self):
        """ Return site-title from portal_url
        """
        portal_url = getToolByName(self.context, 'portal_url')
        return portal_url.getPortalObject().Title()

    def _get_html_body(self, options):
        """ Return the html body of the email
        """
        html_view = self.context.unrestrictedTraverse(
            '@@usermanagament_notify_user')

        html_body = html_view(**options)

        return html_body.encode('utf-8')

    def _add_statusmessage(self, msg, mtype):
        """ Add a statusmessage
        """
        IStatusMessage(self.request).addStatusMessage(msg, type=mtype)

    def _get_subject(self, site_title):
        """Returns the translated subject of the email.
        """
        return translate(u'usermanagement_mail_notification_subject',
                         domain='ftw.usermanagement',
                         context=self.request,
                         default=u'Welcome on ${title}',
                         mapping=dict(title=site_title))



from setuptools import setup, find_packages
import os

version = '1.8.3'
maintainer = 'Mathias Leimgruber'
tests_require = [
    'ftw.testing',
    'ftw.tabbedview',
    'ftw.table',
    'plone.testing',
    'plone.app.testing',
    ]

setup(name='ftw.usermanagement',
      version=version,
      description='An advanced view for managing users in plone.',
      long_description=open('README.rst').read() + '\n' + \
          open(os.path.join('docs', 'HISTORY.txt')).read(),

      # Get more strings from
      # http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        'Framework :: Plone',
        'Framework :: Plone :: 4.1',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Programming Language :: Python',
        ],

      keywords='ftw usermanagement user management view plone',
      author='4teamwork GmbH',
      author_email='mailto:info@4teamwork.ch',
      maintainer=maintainer,
      url='https://github.com/4teamwork/ftw.usermanagement/',
      license='GPL2',

      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['ftw'],
      include_package_data=True,
      zip_safe=False,

      install_requires=[
        'setuptools',
        'ftw.table',
        'ftw.tabbedview',
        'plone.principalsource',
        'collective.js.jqueryui',
        'collective.js.ui.multiselect',
        # -*- Extra requirements: -*-
        ],

      tests_require=tests_require,
      extras_require=dict(tests=tests_require),
      entry_points='''
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      ''',
      )



# coding=UTF-8
from ftw.testing import MockTestCase
from mocker import ANY, KWARGS
from ftw.usermanagement.testing import USERMANAGEMENT_ZCML_LAYER
from ftw.usermanagement.browser.group.group_add import GroupAdd
from Products.statusmessages.interfaces import IStatusMessage
from zope.interface import Interface


class ValidateGroupTests(MockTestCase):

    layer = USERMANAGEMENT_ZCML_LAYER

    def setUp(self):
        super(ValidateGroupTests, self).setUp()

        self.request = {}
        self.context = self.stub()

        self.rtool = self.mocker.mock(count=False)
        self.mock_tool(self.rtool, 'portal_registration')
        self.expect(self.rtool.isMemberIdAllowed('invalid')).result(False)
        self.expect(self.rtool.isMemberIdAllowed('valid')).result(True)

        self.statusmsg = self.mocker.mock(count=False)
        self.message_cache = self.create_dummy()
        self.expect(self.statusmsg(ANY).addStatusMessage(ANY, type=ANY)).call(
            lambda msg, type: setattr(self.message_cache, type, msg))
        self.mock_adapter(self.statusmsg, IStatusMessage, (Interface, ))

    def test_invalid_id(self):

        self.replay()

        result = GroupAdd(self.context, self.request).validate_group('invalid')

        self.assertEqual(result, False)
        self.assertEqual(
            self.message_cache.error,
            u"The group id you selected is already"
            "in use or is not valid. Please choose another.")

    def test_valid_id(self):

        self.replay()

        result = GroupAdd(self.context, self.request).validate_group('valid')

        self.assertEqual(result, True)


class AddGroupTests(MockTestCase):

    layer = USERMANAGEMENT_ZCML_LAYER

    def setUp(self):
        super(AddGroupTests, self).setUp()

        self.request = {}
        self.context = self.stub()

        self.gtool = self.mocker.mock(count=False)
        self.mock_tool(self.gtool, 'portal_groups')
        self.group_pool = {}
        self.expect(self.gtool.addGroup(ANY, KWARGS)).call(
            lambda x, title: self.group_pool.update({x: title}))

        self.statusmsg = self.mocker.mock(count=False)
        self.message_cache = self.create_dummy()
        self.expect(self.statusmsg(ANY).addStatusMessage(ANY, type=ANY)).call(
            lambda msg, type: setattr(self.message_cache, type, msg))
        self.mock_adapter(self.statusmsg, IStatusMessage, (Interface, ))

    def test_create_group_invalid_id(self):

        group_add = self.mocker.patch(GroupAdd(self.context, self.request))
        self.expect(group_add.validate_group(ANY)).result(False)

        self.replay()

        result = group_add.create_group('invalid_id', 'title')

        self.assertEqual(result, False)
        self.assertEqual(self.group_pool, {})

    def test_create_group_no_title(self):

        group_add = self.mocker.patch(GroupAdd(self.context, self.request))
        self.expect(group_add.validate_group(ANY)).result(True)

        self.replay()

        result = group_add.create_group('group_id', '')

        self.assertTrue

        self.assertEqual(result, True)
        self.assertEqual(self.message_cache.info, u'text_group_created')
        self.assertEqual(self.group_pool, {'group_id': 'group_id'})

    def test_create_group(self):

        group_add = self.mocker.patch(GroupAdd(self.context, self.request))
        self.expect(group_add.validate_group(ANY)).result(True)

        self.replay()

        result = group_add.create_group('group_id', 'Ädmin')

        self.assertTrue

        self.assertEqual(result, True)
        self.assertEqual(self.message_cache.info, u'text_group_created')
        self.assertEqual(self.group_pool, {'group_id': 'Ädmin'})



# coding=UTF-8
from ftw.testing import MockTestCase
from mocker import ANY
from ftw.usermanagement.testing import USERMANAGEMENT_ZCML_LAYER
from ftw.usermanagement.browser.user.user_membership import UserMembership
from Products.statusmessages.interfaces import IStatusMessage
from zope.interface import Interface


class DisplayGroupsTests(MockTestCase):

    layer = USERMANAGEMENT_ZCML_LAYER

    def setUp(self):
        super(DisplayGroupsTests, self).setUp()

        self.request = {}

        self.context = self.stub()
        self.expect(self.context.REQUEST).result(self.request)

        self.user_membership = self.mocker.patch(
            UserMembership(self.context, self.request))
        self.expect(self.user_membership.get_groups(ANY)).call(lambda x: x)

        self.group_1 = self.mocker.mock(count=False)
        self.expect(self.group_1.getId()).result('group_1')
        self.expect(self.group_1.getGroupTitleOrName()).result('Groüp 1')

        self.group_2 = self.mocker.mock(count=False)
        self.expect(self.group_2.getId()).result('group_2')
        self.expect(self.group_2.getGroupTitleOrName()).result('Group 2')

        self.group_3 = self.mocker.mock(count=False)
        self.expect(self.group_3.getId()).result('AuthenticatedUsers')
        self.expect(self.group_3.getGroupTitleOrName()).result('Group 2')

    def test_no_groups(self):

        self.expect(
            self.user_membership.membershipSearch(searchUsers=ANY)).result([])

        self.replay()

        result = self.user_membership.get_display_groups([])

        self.assertEquals(result, [])

    def test_with_current_groups(self):

        self.expect(
            self.user_membership.membershipSearch(searchUsers=ANY)).result(
                [self.group_1, self.group_2])

        self.replay()

        result = self.user_membership.get_display_groups(['group_2'])

        self.assertEquals(
            result,
            [{
                'name': 'group_1',
                'title': 'Groüp 1',
                'is_member_of': False,
            },
            {
                'name': 'group_2',
                'title': 'Group 2',
                'is_member_of': True,
            }],
        )

    def test_no_current_groups(self):

        self.expect(
            self.user_membership.membershipSearch(searchUsers=ANY)).result(
                [self.group_1, self.group_2])

        self.replay()

        result = self.user_membership.get_display_groups([])

        self.assertEquals(
            result,
            [{
                'name': 'group_1',
                'title': 'Groüp 1',
                'is_member_of': False,
            },
            {
                'name': 'group_2',
                'title': 'Group 2',
                'is_member_of': False,
            }],
        )

    def test_authenthicated_group(self):

        self.expect(
            self.user_membership.membershipSearch(searchUsers=ANY)).result(
                [self.group_3])

        self.replay()

        result = self.user_membership.get_display_groups([])

        self.assertEquals(result, [])


class ReplaceGroupTests(MockTestCase):

    layer = USERMANAGEMENT_ZCML_LAYER

    def setUp(self):
        super(ReplaceGroupTests, self).setUp()

        self.request = {}

        self.context = self.stub()
        self.expect(self.context.REQUEST).result(self.request)

        self.gtool = self.mocker.mock(count=False)
        self.mock_tool(self.gtool, 'portal_groups')

        self.statusmsg = self.mocker.mock(count=False)
        self.message_cache = self.create_dummy()
        self.expect(self.statusmsg(ANY).addStatusMessage(ANY, type=ANY)).call(
            lambda msg, type: setattr(self.message_cache, type, msg))
        self.mock_adapter(self.statusmsg, IStatusMessage, (Interface, ))

        self.user_membership = self.mocker.patch(
            UserMembership(self.context, self.request))
        self.expect(self.user_membership.get_groups(ANY)).call(lambda x: x)

        self.removed_groups = []
        self.added_groups = []

        self.expect(self.gtool.removePrincipalFromGroup(
            ANY, ANY, ANY)).call(lambda x, y, z: self.removed_groups.append(x))
        self.expect(self.gtool.addPrincipalToGroup(
            ANY, ANY, ANY)).call(lambda x, y, z: self.added_groups.append(x))

    def test_nothing(self):

        self.replay()

        result = self.user_membership.replace_groups([], [])

        self.assertEquals(result, True)
        self.assertEquals(self.message_cache.info, u'Changes made.')
        self.assertTrue(len(self.removed_groups) == 0)
        self.assertTrue(len(self.added_groups) == 0)

    def test_add(self):

        self.replay()

        result = self.user_membership.replace_groups(
            [], ['group1', 'group2'])

        self.assertEquals(result, True)
        self.assertEquals(self.message_cache.info, u'Changes made.')
        self.assertTrue(len(self.removed_groups) == 0)
        self.assertTrue(len(self.added_groups) == 2)

    def test_remove(self):

        self.replay()

        result = self.user_membership.replace_groups(
            ['group1', 'group2'], [])

        self.assertEquals(result, True)
        self.assertEquals(self.message_cache.info, u'Changes made.')
        self.assertTrue(len(self.removed_groups) == 2)
        self.assertTrue(len(self.added_groups) == 0)

    def test_remove_and_add(self):

        self.replay()

        result = self.user_membership.replace_groups(
            ['group1', 'group2'], ['user3', 'user4'])

        self.assertEquals(result, True)
        self.assertEquals(self.message_cache.info, u'Changes made.')
        self.assertTrue(len(self.removed_groups) == 2)
        self.assertTrue(len(self.added_groups) == 2)

    def test_leave_all(self):

        self.replay()

        result = self.user_membership.replace_groups(
            ['group1', 'group2'], ['group1', 'group2'])

        self.assertEquals(result, True)
        self.assertEquals(self.message_cache.info, u'Changes made.')
        self.assertTrue(len(self.removed_groups) == 0)
        self.assertTrue(len(self.added_groups) == 0)

    def test_authenthicated_group(self):

        self.replay()

        result = self.user_membership.replace_groups(
            ['AuthenticatedUsers'], [])

        self.assertEquals(result, True)
        self.assertEquals(self.message_cache.info, u'Changes made.')
        self.assertTrue(len(self.removed_groups) == 0)
        self.assertTrue(len(self.added_groups) == 0)


class CallTests(MockTestCase):

    layer = USERMANAGEMENT_ZCML_LAYER

    def setUp(self):
        super(CallTests, self).setUp()

        self.request = {}

        self.context = self.stub()
        self.expect(self.context.REQUEST).result(self.request)

    def test_no_user_id(self):

        request = self.mocker.proxy(self.request)
        self.expect(request.form).result({})

        self.replay()

        result = UserMembership(self.context, request)()

        self.assertEquals(result, 'No user selected')

    def test_form_submitted(self):

        self.request['userid'] = "User_1"
        request = self.mocker.proxy(self.request)
        self.expect(request.form).result({'form.submitted': True})

        user_membership = self.mocker.patch(
            UserMembership(self.context, request))
        self.expect(
            user_membership.replace_groups(ANY, ANY)).result('replace')

        self.replay()

        result = user_membership()

        self.assertEquals(result, 'replace')

    def test_form_not_submitted(self):

        self.request['userid'] = "User_1"
        request = self.mocker.proxy(self.request)
        self.expect(request.form).result({'form.submitted': False})

        user_membership = self.mocker.patch(
            UserMembership(self.context, request))
        self.expect(
            user_membership.render(ANY)).result('render')

        self.replay()

        result = user_membership()

        self.assertEquals(result, 'render')


class GetUsersTests(MockTestCase):

    layer = USERMANAGEMENT_ZCML_LAYER

    def setUp(self):
        super(GetUsersTests, self).setUp()

        self.request = {}

        self.context = self.stub()
        self.expect(self.context.REQUEST).result(self.request)

        self.gtool = self.mocker.mock(count=False)
        self.mock_tool(self.gtool, 'portal_groups')

        self.mtool = self.mocker.mock(count=False)
        self.mock_tool(self.mtool, 'portal_membership')
        self.expect(self.mtool.getMemberById(ANY)).call(lambda x: x)

        self.group_1 = self.mocker.mock(count=False)
        self.expect(self.group_1.getGroupTitleOrName()).result('member group')
        self.expect(self.group_1.getId()).result('group_1')

        self.group_2 = self.mocker.mock(count=False)
        self.expect(self.group_2.getGroupTitleOrName()).result('admin group')
        self.expect(self.group_2.getId()).result('group_2')

        self.group_3 = self.mocker.mock(count=False)
        self.expect(self.group_3.getGroupTitleOrName()).result('user group')
        self.expect(self.group_3.getId()).result('group_3')

    def test_no_groups(self):

        self.expect(self.gtool.getGroupsForPrincipal(ANY)).result([])

        self.replay()

        um = UserMembership(self.context, self.request)
        result = um.get_groups('no_group')

        # No users
        self.assertEquals(result, [])

    def test_with_members(self):

        self.expect(self.gtool.getGroupsForPrincipal(ANY)).result(
            ['member_group', 'admin_group', 'user_group'])

        self.expect(
            self.gtool.getGroupById('member_group')).result(self.group_1)
        self.expect(
            self.gtool.getGroupById('admin_group')).result(self.group_2)
        self.expect(
            self.gtool.getGroupById('user_group')).result(self.group_3)

        self.replay()

        um = UserMembership(self.context, self.request)
        result = um.get_groups('group_id')

        # Sorted list with userids
        self.assertEquals(result, ['group_2', 'group_1', 'group_3'])



from Products.CMFCore.utils import getToolByName
from ftw.usermanagement import user_management_factory as _
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.statusmessages.interfaces import IStatusMessage
from plone.app.controlpanel.usergroups import UsersGroupsControlPanelView


class GroupMembership(UsersGroupsControlPanelView):
    """Provides another way to assign users to a group
    """

    template = ViewPageTemplateFile('group_membership.pt')

    def __call__(self):
        """ Check the given request parameters and call the
        correct functions
        """
        form = self.request.form
        group_id = self.request.get('group_id', None)
        users = form.get('new_users', [])

        if not group_id:
            # Fallback
            return 'No group selected'

        if not form.get('form.submitted', False):
            return self.render(group_id)

        return self.replace_group_members(group_id, users)

    def render(self, group_id):
        """ Renders the popup to assign new users
        """
        users = self.get_display_users(group_id)
        return self.template(users=users)

    def replace_group_members(self, group_id, users):
        """ Replace the assigned users in the users-attr
        """
        current_users = self.get_users(group_id)
        gtool = getToolByName(self.context, 'portal_groups')

        # Remove unselected users
        for user in current_users:
            if user in users:
                continue
            gtool.removePrincipalFromGroup(user, group_id, self.request)

        # Add users to groups
        for user in users:
            if user in current_users:
                continue
            gtool.addPrincipalToGroup(user, group_id, self.request)

        IStatusMessage(self.request).addStatusMessage(
            _(u'Changes made.'), type="info")

        return True

    def get_users(self, group_id):
        """ Return a sorted list of users assigned to a group
        """
        gtool = getToolByName(self.context, 'portal_groups')
        mtool = getToolByName(self.context, 'portal_membership')

        def sort_users(user):
            member = mtool.getMemberById(user)
            if not member:
                return user
            fullname = member.getProperty('fullname', '')
            if not fullname:
                return user
            return fullname

        group_members = gtool.getGroupMembers(group_id)
        clean_members = []
        for member in group_members:
            user_info = mtool.getMemberInfo(member)
            if not user_info:
                continue
            clean_members.append(member)
        clean_members.sort(key=sort_users)

        return filter(None, group_members)

    def get_display_users(self, group_id):
        """ Returns a list of dicts with name, title and is_member_of
        """
        users = []
        current_users = self.get_users(group_id)

        for m in self.membershipSearch(searchGroups=False):
            fullname = m.getProperty('fullname', m.getUserId())
            users.append(dict(
                userid=m.getUserId(),
                name=fullname and fullname or m.getUserId(),
                is_member_of=m.getUserId() in current_users))

        return users
